@@ -29,6 +29,7 @@
 
 #include <stdarg.h>
 #include <glib/gtypes.h>
+#include <glib/gmacros.h>
 
 /* Suppress warnings when GCC is in -pedantic mode and not -std=c99
  */
@@ -38,7 +39,7 @@
 
 G_BEGIN_DECLS
 
-/* calculate a string size, guarranteed to fit format + args.
+/* calculate a string size, guaranteed to fit format + args.
  */
 gsize	g_printf_string_upper_bound (const gchar* format,
 				     va_list	  args);
@@ -99,6 +100,13 @@ GLogLevelFlags  g_log_set_fatal_mask    (const gchar    *log_domain,
                                          GLogLevelFlags  fatal_mask);
 GLogLevelFlags  g_log_set_always_fatal  (GLogLevelFlags  fatal_mask);
 
+/* internal */
+void	_g_log_fallback_handler	(const gchar   *log_domain,
+				 GLogLevelFlags log_level,
+				 const gchar   *message,
+				 gpointer       unused_data);
+
+
 #ifndef G_LOG_DOMAIN
 #define G_LOG_DOMAIN    ((gchar*) 0)
 #endif  /* G_LOG_DOMAIN */
@@ -115,6 +123,9 @@ GLogLevelFlags  g_log_set_always_fatal  (GLogLevelFlags  fatal_mask);
 #define g_warning(...)  g_log (G_LOG_DOMAIN,         \
                                G_LOG_LEVEL_WARNING,  \
                                __VA_ARGS__)
+#define g_debug(...)    g_log (G_LOG_DOMAIN,         \
+                               G_LOG_LEVEL_DEBUG,    \
+                               __VA_ARGS__)
 #elif defined(G_HAVE_GNUC_VARARGS)
 #define g_error(format...)      g_log (G_LOG_DOMAIN,         \
                                        G_LOG_LEVEL_ERROR,    \
@@ -128,6 +139,9 @@ GLogLevelFlags  g_log_set_always_fatal  (GLogLevelFlags  fatal_mask);
 #define g_warning(format...)    g_log (G_LOG_DOMAIN,         \
                                        G_LOG_LEVEL_WARNING,  \
                                        format)
+#define g_debug(format...)      g_log (G_LOG_DOMAIN,         \
+                                       G_LOG_LEVEL_WARNING,  \
+                                       format)
 #else   /* no varargs macros */
 static void
 g_error (const gchar *format,
@@ -165,6 +179,15 @@ g_warning (const gchar *format,
   g_logv (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING, format, args);
   va_end (args);
 }
+static void
+g_debug (const gchar *format,
+         ...)
+{
+  va_list args;
+  va_start (args, format);
+  g_logv (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, format, args);
+  va_end (args);
+}
 #endif  /* !__GNUC__ */
 
 typedef void    (*GPrintFunc)           (const gchar    *string);
@@ -185,16 +208,16 @@ GPrintFunc      g_set_printerr_handler  (GPrintFunc      func);
 
 #ifdef G_DISABLE_ASSERT
 
-#define g_assert(expr)		G_STMT_START{ }G_STMT_END
-#define g_assert_not_reached()	G_STMT_START{ }G_STMT_END
+#define g_assert(expr)		G_STMT_START{ (void)0; }G_STMT_END
+#define g_assert_not_reached()	G_STMT_START{ (void)0; }G_STMT_END
 
 #else /* !G_DISABLE_ASSERT */
 
 #ifdef __GNUC__
 
 #define g_assert(expr)			G_STMT_START{		\
-     if (expr) { } else						\
-       g_log (G_LOG_DOMAIN,					\
+     if G_LIKELY(expr) { } else 				\
+        g_log (G_LOG_DOMAIN,					\
 	      G_LOG_LEVEL_ERROR,				\
 	      "file %s: line %d (%s): assertion failed: (%s)",	\
 	      __FILE__,						\
@@ -235,8 +258,8 @@ GPrintFunc      g_set_printerr_handler  (GPrintFunc      func);
 
 #ifdef G_DISABLE_CHECKS
 
-#define g_return_if_fail(expr)			G_STMT_START{ }G_STMT_END
-#define g_return_val_if_fail(expr,val)		G_STMT_START{ }G_STMT_END
+#define g_return_if_fail(expr)			G_STMT_START{ (void)0; }G_STMT_END
+#define g_return_val_if_fail(expr,val)		G_STMT_START{ (void)0; }G_STMT_END
 #define g_return_if_reached()			G_STMT_START{ return; }G_STMT_END
 #define g_return_val_if_reached(val)		G_STMT_START{ return (val); }G_STMT_END
 
@@ -245,7 +268,7 @@ GPrintFunc      g_set_printerr_handler  (GPrintFunc      func);
 #ifdef __GNUC__
 
 #define g_return_if_fail(expr)		G_STMT_START{			\
-     if (expr) { } else							\
+     if G_LIKELY(expr) { } else       					\
        {								\
 	 g_log (G_LOG_DOMAIN,						\
 		G_LOG_LEVEL_CRITICAL,					\
@@ -258,7 +281,7 @@ GPrintFunc      g_set_printerr_handler  (GPrintFunc      func);
        };				}G_STMT_END
 
 #define g_return_val_if_fail(expr,val)	G_STMT_START{			\
-     if (expr) { } else							\
+     if G_LIKELY(expr) { } else						\
        {								\
 	 g_log (G_LOG_DOMAIN,						\
 		G_LOG_LEVEL_CRITICAL,					\
