@@ -934,10 +934,7 @@ g_get_any_init (void)
         do
           {
             g_free (buffer);
-	    /* we allocate 6 extra bytes to work around a bug in 
-	     * Mac OS < 10.3. See #156446
-             */
-            buffer = g_malloc (bufsize + 6);
+            buffer = g_malloc (bufsize);
 	    errno = 0;
 	    
 #    ifdef HAVE_POSIX_GETPWUID_R
@@ -1239,11 +1236,11 @@ g_nullify_pointer (gpointer *nullify_location)
 gchar *
 g_get_codeset (void)
 {
-  const gchar *charset;
+  gchar *charset;
 
   g_get_charset (&charset);
 
-  return strdup (charset);
+  return charset;
 }
 
 #ifdef ENABLE_NLS
